<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Exception;
use Geral\Entities\Municipio;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;

/**
 * Class GeraCnabBrasilService
 * @package Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco
 */
class GeraCnabBrasilService extends GeraCnabService{


    CONST TIPO_INS_JUR_CPF = 1;
    CONST TIPO_INS_JUR_CNPJ = 2;
    CONST LOTE_SERVICO_HEADER_ARQUIVO = '0000';
    CONST TIPO_REGISTRO_HEADER_ARQUIVO = '0';
    CONST TIPO_REGISTRO_HEADER_LOTE = '1';
    CONST TIPO_REGISTRO_DETALHE = '3';
    CONST TIPO_REGISTRO_TRAILLER_LOTE ='5';
    CONST TIPO_REGISTRO_TRAILLER_ARQUIVO = '9';
    CONST ID_REMESSA = '1';
    CONST ID_RETORNO = '2';
    CONST VERSAO_LAYOUT_ARQUIVO = '030';
    CONST VERSAO_LAYOUT_LOTE = '020';
    CONST TIPO_SERVICO_PAGAMENTO_FORNECEDOR = '20';
    CONST CREDITO = 'C';
    CONST MOEDA = 'BRL';
    CONST LOTE_DE_SERVICO_TRAILLER_ARQUIVO = '9999';

    private $formaPgto;
    private $num_header_lote = 0; // Sequencial de Header de Lote
    private $seqRegLote = 0; // Sequencial do Lote
    private $qtdRegiLote = 0;  //Qtde de registro do lote
    private $qtdRegiArq = 0; //Quantidade de registro do arquivo
    private $qtLote = 0; //Quantidade de Lotes no arquivo
    private $num_lote = 0; // Sequencial incrementado em 1 para cada lote no arquivo
    private $vrPagtLote  = 0; //Somatório dos registros com tipo de movimento 000 (inclusão)

    /**
     * @param Cnabpag $remessa
     * @throws Exception
     */
    public function __construct(Cnabpag $remessa){

        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;

            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = Util::removeMascaraInscricao($this->filial->getNrinsjurfili());
                $this->nomeempresa = $this->filial->getNmrazsocfili();
                $this->dsEndeEF = $this->filial->getEndefiliprinc()->getDsendefili();
                $this->muniEF = $this->filial->getEndefiliprinc()->getMunicipio()->getNmmunicipio();
                $this->cepEF = $this->filial->getEndefiliprinc()->getNrcepfili();
                $this->estadoEF = $this->filial->getEndefiliprinc()->getSgestado();

            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = Util::removeMascaraInscricao($this->empresa->getNrinsjurempr());
                $this->nomeempresa = $this->empresa->getNmrazsocempr();
                $this->dsEndeEF = $this->empresa->getDsendeempr();
                $this->muniEF = $this->empresa->getMunicipio()->getNmmunicipio();
                $this->cepEF = $this->empresa->getNrcepempr();
                $this->estadoEF = $this->empresa->getSgestado();
            }
            /** @var contcorr Contcorr */
            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @return string
     * @throws Exception
     *  //#################################################################//
        //############### REGISTRO HEADER DE ARQUIVO ######################//
        //#################################################################//
     */
    public function geraHeaderArquivo()
    {
        $linha = '';
        $this->qtdRegiArq++; //Quantidade de registro do arquivo

        //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(3)
        $linha .= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do Lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= self::LOTE_SERVICO_HEADER_ARQUIVO;

        //Tipo de registro = Registro Header - Posição: 8 a 8 - Picture: 9(1)
        $linha .= self::TIPO_REGISTRO_HEADER_ARQUIVO;

        //Uso exclusivo FEBRABAN/CNAB -Brancos - Posição: 9 a 17 - Picture: X(9)
        $linha .= str_repeat(' ', 9);

        //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
        $linha .= $this->tipoinscricao;

        //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
        $linha .= str_pad(trim($this->inscricaojuridica), 14, '0', STR_PAD_LEFT);

        //Codigo do Convenio no Banco
        //Posição: 33 a 52
        //Picture: X(20)
        // 09 - Código do Convênio
        // 04 - Código do Produto      "0014"
        // 02 - Carteira               "00"
        // 03 - Variação da Carteira   "000"
        // 02 - Integração SAP         "Vazio"
        $identpag = self::getIdentPag();
        if ($identpag) {
            $linha .= str_pad(Util::campoCnabValido(substr(trim($identpag), 0, 9)), 9, '0', STR_PAD_LEFT) . '0126' . str_repeat(' ', 7);
        } else {
            $linha .= str_repeat('0', 9) . '0126' . str_repeat(' ', 7);
        }

        //Número da agência debitada - Posição: 53 a 57 - Picture: 9(05)
        $linha .= str_pad(substr(trim($this->contcorr->getCdagencia()), 0, -1), 5, '0', STR_PAD_LEFT);

        //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
        $linha .= substr(trim($this->contcorr->getCdagencia()), -1);

        //Nº da C/C Debitada - Posição: 59 a 70 - Picture: 9(12)
        //Obs: A última posição da C/C é reservada para o DAC
        $linha .= str_pad(substr(trim(self::getContaCorrenteSemDigito()), 0, 12), 12, '0', STR_PAD_LEFT);

        //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
        $linha .= substr(self::getDigitoContaCorrente(), 0, 1);

        //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
        $linha .= ' ';

        //Nome da empresa - Posição: 73 a 102 - Picture: X(30)
        $linha .= str_pad(substr(trim(Util::campoCnabValido($this->nomeempresa)), 0, 30), 30, ' ', STR_PAD_RIGHT);

        //Nome do Banco - Posição: 103 a 132 - Picture: X(30)
        $linha .= str_pad(substr(Util::campoCnabValido($this->contcorr->getAgencia()->getBanco()->getNmbanco()),
            0, 30), 30, ' ', STR_PAD_RIGHT);

        //Filler - Posição: 133 a 142 - Picture: X(10)
        $linha .= str_repeat(' ', 10);

        //Código remessa/Retorno (1 = Remessa, 2 = Retorno) - Posição: 143 a 143 - Picture: 9(1)
        $linha .= self::ID_REMESSA;

        //Data da geração do arquivo - Posição: 144 a 151 - Picture: 9(8)
        $dataAtual = DateUtil::getDataAtual();
        $linha .= $dataAtual->format('dmY');

        //Hora de geração do arquvio - Posição: 152 a 157 - Picture 9(6)
        $linha .= $dataAtual->format('His');

        //Número de sequência do arquivo - Posição: 158 a 163 - Picture: 9(06)
        $linha .= substr($this->remessa->getNrseqremessapag(), -6);

        //Número da versão do layout do arquivo - Posição: 164 a 166 - Picture: 9(03)
        $linha .= self::VERSAO_LAYOUT_ARQUIVO;

        //Densidade de gravação do arquivo - Posição: 167 a 171 - Picture: 9(5)
        $linha .= str_repeat('0', 5);

        // Reservado para o Banco
        // POSIÇÃO: 172 a 191
        // PICTURE: X(20)
        $linha .= str_repeat(' ', 20);

        // Reservado para a Empresa
        // POSIÇÃO: 192 a 211
        // PICTURE: X(20)
        $linha .= str_repeat(' ', 20);

        // Brancos
        // POSIÇÃO: 212 a 222
        // PICTURE: X(11)
        $linha .= str_repeat(' ', 11);

        // Conbranca s/ papel
        // POSIÇÃO: 223 a 225
        // PICTURE: X(3)
        $linha .= 'CSP';

        // Uso exclusivo das Vans
        // POSIÇÃO: 226 a 228
        // PICTURE: X(3)
        $linha .= '000';

        // POSIÇÃO: 229 a 240
        // PICTURE: X(12)
        $linha.= str_repeat(' ',12);

        return $linha."\r\n";
    }

    /**
     * @param $cdbanco
     * @internal param $formaLancamento
     * @return mixed
     * /#################################################################//
       //################ REGISTRO HEADER DE LOTE  ######################//
      //#################################################################//
     */
    public function geraHeaderLote($cdbanco, $boletos = null)
    {

        $this->seqRegLote = 0;
        $this->vrPagtLote = 0;
        $this->qtdRegiLote = 0;
        $this->qtdRegiLote++;
        $this->qtdRegiArq++;

        $linha = '';

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do Lote (Lote de identificação de pagtos - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad(++$this->num_lote,4,'0',STR_PAD_LEFT);

        //Tipo de registro = registro Header de lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_HEADER_LOTE;

        //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(1)
        $linha.= 'C';

        //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(2)
        $linha.= self::TIPO_SERVICO_PAGAMENTO_FORNECEDOR;

        //********************************************************
        //*********** Forma de Lançamento*************************
        //********************************************************

        //Forma de Lançamento- Posição: 12 a 13 - Picture: 9(2)
        $this->formaPgto = self::getFormaPagto($cdbanco, $boletos);
        $linha.= $this->formaPgto;

        //Nº da versão do layoute do lote - Posição: 14 a 16 - Picture: 9(3)
        $linha.= self::VERSAO_LAYOUT_LOTE;

        //Brancos - Posição: 17 a 17 - Picture: X(1)
        $linha.= ' ';

        //Tipo inscrição filial debitada - Posição: 18 a 18 - Picture: 9(1)
        $linha.= $this->tipoinscricao;

        //CGC filial debitada  - Posição: 19 a 32 - Picture: 9(14)
        $linha.= $this->inscricaojuridica;

        //Codigo do Convenio no Banco
        //Posição: 33 a 52
        //Picture: X(20)
        // 09 - Código do Convênio
        // 04 - Código do Produto      "0014"
        // 02 - Carteira               "00"
        // 03 - Variação da Carteira   "000"
        // 02 - Integração SAP         "Vazio"
        $identpag  = self::getIdentPag();
        if($identpag)
            $linha.= str_pad(Util::campoCnabValido(substr(trim($identpag),0,9)),9,'0'). '0126'. str_repeat(' ', 7);
        else
            $linha.= str_repeat('0', 9). '0126'. str_repeat(' ', 7);

        //Número da agência debitada - Posição: 53 a 57 - Picture: 9(05)
        $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()), 0, 4), 5,'0',STR_PAD_LEFT);

        //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
        $linha.= substr(trim($this->contcorr->getCdagencia()), -1);

        //Nº da C/C Debitada - Posição: 59 a 70 - Picture: 9(12)
        //Obs: A última posição da C/C é reservada para o DAC
        $linha.= str_pad(Util::campoCnabValido(substr(self::getContaCorrenteSemDigito(), 0, 12)), 12,'0',STR_PAD_LEFT);

        //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
        $linha.= trim(Util::campoCnabValido(substr($this->contcorr->getCdcontcorr(),-1)));

        //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
        $linha.= ' ';

        //Nome da empresa/filial debitada - Posição: 73 a 102 - Picture: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa, 0, 30)), 30,' ',STR_PAD_RIGHT);

        //Mensagem - Posição: 103 a 142 - Picture: X(40)
        $linha.= str_repeat(' ', 40);

        //Endereço da empresa - Posição: 143 a 172 - Picture: X(30)
        $linha.= str_pad(Util::campoCnabValido(substr($this->dsEndeEF, 0, 30)), 30,' ',STR_PAD_RIGHT);

        //Número do Local - Posição: 173 a 177 - Picture: 9(5)
        $linha.= str_repeat('0', 5);

        //Complemento (Casa, Apto, sala, etc) - Posição: 178 a 192 - Picture: X(15)
        $linha.= str_repeat(' ', 15);
        //Nome da cidade - Posição: 193 a 212 - Picture: X(20)
        $linha.= str_pad(Util::campoCnabValido(substr($this->muniEF, 0, 20)), 20,' ',STR_PAD_RIGHT);

        //CEP - Posição: 213 a 220 - Picture: 9(8)
        $linha.= str_pad(Util::campoCnabValido(substr($this->cepEF, 0, 8)), 8,' ',STR_PAD_RIGHT);

        //Sigla do Estado - Posição: 221 a 222 - Picture: X(2)
        $linha.= str_pad(Util::campoCnabValido($this->estadoEF), 2,' ',STR_PAD_RIGHT);

        //Brancos - Posição: 223 a 230 - Picture: X(8)
        $linha.= str_repeat(' ', 8);

        //Ocorrências p/ retorno: 231 a 240 - Picture: X(10)
        $linha.= str_repeat(' ', 10);

        return $linha."\r\n";
    }

    /**
     * @param Boletpagar $boleto
     * @param $cdbanco
     * @return string
     */
    private function geraSegmentoAJ(Boletpagar $boleto, $cdbanco)
    {

        $linha = '';
        $this->qtdRegiLote++;//Qtde de registro do lote
        $this->qtdRegiArq++; //Quantidade de registro do arquivo

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha .= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha .= self::TIPO_REGISTRO_DETALHE;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(1)
        if (($this->formaPgto != '30') and ($this->formaPgto != '31'))
            $linha.= 'A';
        else
            $linha.= 'J';

        //Tipo de movimento - Posição: 15 a 15 - Picture: 9(1)
        if ($this->remessa->getIdopecnabpag() == 'E')
            $linha.= '9';
        else{
            $linha.= '0';
            //Somatório dos registros com tipo de movimento = 000 (inclusão)
            $this->vrPagtLote+= round($boleto->getVrfinalbpag(), 2);
        }

        //Codigo de instrução para alteração - Posição:16 a 17 -Picture:9(2)
        if ($this->remessa->getIdopecnabpag() == 'E')
           $linha .= '99'; //Exclusão
        else {
            if ($this->remessa->getIdopecnabpag() == 'I')
                $linha .= '00'; //Inclusão
            else
                $linha .= '09';   // Bloqueio
        }

        //---------------------------------------------------------------------------------------------------------
        if (($this->formaPgto != '30') and ($this->formaPgto != '31')) {
            //Código da câmara centralizadora - Posição: 18 a 20 - Picture: 9(3)
            if ($this->formaPgto == '03') {
                if (round($boleto->getVrfinalbpag(), 2) > 5000)
                    $linha .= '018';
                else
                    $linha .= '700';
            } else {
                $linha .= '000';
            }

            //Código do Banco - Posição: 21 a 23 - Picture: 9(3)
            $linha .= str_pad(substr($cdbanco, 0, 3), 3, '0', STR_PAD_LEFT);

            if ($cdbanco == self::CODIGO_COMPENSACAO_BRASIL) {

                if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
                    //Numero da agencia creditada - Posição: 25 a 28 - Picture: 9(4)
                    $linha .= str_pad(substr(trim($boleto->getReducont()->getCdagencia()), 0, 4), 5, '0', STR_PAD_LEFT);

                    //Digito verificador da Agência- Posição: 29 a 29 - Picture: 9(1)
                    $linha .= str_pad(substr(trim($boleto->getReducont()->getCdagencia()), -1), 1, ' ', STR_PAD_LEFT);
                } else {
                    //Numero da agencia creditada - Posição: 25 a 28 - Picture: 9(5)
                    $linha .= str_pad(substr(trim($boleto->getCdagenbolp()), 0, 4), 5, '0', STR_PAD_LEFT);

                    //Digito verificador da Agência- Posição: 29 a 29 - Picture: 9(1)
                    $linha .= str_pad(substr(trim($boleto->getCdagenbolp()), -1, 1), 1, ' ', STR_PAD_LEFT);
                }

                //Numero da Conta Corrente - Posição: 30 a 41 - Picture: 9(12)
                // CAMPO: Conta Favorecido (Sem DV)
                // POSIÇÃO: 30 a 41
                // PICTURE: 9(10)
                if (strpos($boleto->getCdccorbolp(), '-') === false) {
                    $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdccorbolp(), 0, 7))), 12, '0', STR_PAD_LEFT);
                    //DV C/C - Posição: 42 a 42 - Picture: X(1)
                    $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdccorbolp(), -1))), 1, ' ', STR_PAD_LEFT);
                } else {
                    $linha .= substr(str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(), 0,
                            strpos($boleto->getCdccorbolp(), '-'))),
                        12, '0', STR_PAD_LEFT), -12);
                    //DV C/C - Posição: 42 a 42 - Picture: X(1)
                    $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                        (strpos($boleto->getCdccorbolp(), '-') + 1), 1))), 1, ' ', STR_PAD_LEFT);
                }
                //DV Agência e C/C - Posição: 43 a 43 - Picture: X(1)
                $linha .= ' ';
            } else {
                //Ag mantenedora da cta do favor - Posição: 24 a 28 - Picture: 9(5)
                if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
                    $linha .= str_pad(substr($boleto->getReducont()->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT);

                    //Digito verificador da Agência- Posição: 29 a 29 - Picture: 9(1)
                    $linha .= str_pad(substr(trim($boleto->getReducont()->getCdagencia()), -1), 1, ' ', STR_PAD_LEFT);
                } else {
                    if (strpos($boleto->getCdagenbolp(), '-') === false) {
                        $linha .= str_pad(substr($boleto->getCdagenbolp(), 0, -1), 5, '0', STR_PAD_LEFT);
                        $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdagenbolp(), -1, 1))), 1, ' ', STR_PAD_LEFT);
                    } else {
                        $linha .= str_pad(substr($boleto->getCdagenbolp(), 0, strpos($boleto->getCdccorbolp(), '-')),
                            5, '0', STR_PAD_LEFT);
                        $linha .= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdagenbolp(),
                            (strpos($boleto->getCdagenbolp(), '-') + 1), 1))), 1, ' ', STR_PAD_LEFT);
                    }

                }
                //Numero da C/C creditada - Posição: 30 a 41 - Picture: 9(12)
                if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
                    $linha .= str_repeat('0', 12);
                    $digitoConta = '0';
                } else {
                    if (strpos($boleto->getCdccorbolp(), '-') === false) {
                        $linha .= str_pad(trim(substr($boleto->getCdccorbolp(), 0, -1)), 12, '0', STR_PAD_LEFT);
                        $digitoConta = trim(Util::campoCnabValido(substr($boleto->getCdccorbolp(), -1)));
                    } else {
                        $linha .= substr(str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(), 0,
                                strpos($boleto->getCdccorbolp(), '-'))),
                            12, '0', STR_PAD_LEFT), -12);
                        $digitoConta = trim(substr($boleto->getCdccorbolp(), (strpos($boleto->getCdccorbolp(), '-') + 1), 1));
                    }
                }
                //DV Conta Destino - Posição: 42 a 42 - Picture X(1)
                $linha .= $digitoConta;

                //DV Agência/ Conta Destino - Posição: 43 a 43 - Picture X(1)
                $linha .= ' ';
            }

            //Nome do favorecido - Posição: 44 a 73 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30,' ',STR_PAD_RIGHT);

            //Número do documento atribuído pela empresa - Posição: 74 a 93 - Picture: X(20)
            $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );

            //Data do pagamento - Posição: 94 a 101 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Tipo da Moeda - Posição: 102 a 104 - Picture: X(3)
            $linha.= self::MOEDA;

            //Quantidade da moeda - Posição: 105 a 119 - Picture: 9(15)
            $linha.= str_repeat('0', 15);

            //Valor do pagto - Posição: 120 a 134 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Nosso número - Posição: 135 a 149 - Picture: X(15)
            if($this->remessa->getIdopecnabpag() == self::EXCLUSAO)
                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getNrretcnabpag(),0,15)),15,' ',STR_PAD_RIGHT);
            else $linha.= str_repeat(' ', 15);

            //Brancos - Posição: 150 a 154 - Picture: X(5)
            $linha.= str_repeat(' ', 5);

            //Data efetiva - Posição: 155 a 162 - Picture: 9(8)
            $linha.= str_repeat('0', 8);

            //Valor efetiva - Posição: 163 a 177 - Picture: 9(15)
            $linha.= str_repeat('0', 15);

            //Finalidade Detalhe - Posição: 178 a 217 - Picture: X(40)
            $linha.=  str_repeat(' ', 40);

            //Brancos - Posição: 218 a 229 - Picture: X(12)
            $linha.=  str_repeat(' ', 12);

            //Aviso - Posição: 230 a 230 - Picture: X(1)
            $linha.= '0';
        }
        else{

            if ($boleto->getCdbarrapag()) {
                $cdbanco = str_pad(substr($boleto->getCdbarrapag(), 0, 3), 3, '0', STR_PAD_LEFT);
                $cdmoeda = str_pad(substr($boleto->getCdbarrapag(), 3, 1), 1, '0', STR_PAD_LEFT);
                $cddigito = str_pad(substr($boleto->getCdbarrapag(), 4, 1), 1, '0', STR_PAD_LEFT);
                $valor = str_pad(substr($boleto->getCdbarrapag(), 5, 14), 14, '0', STR_PAD_LEFT);
                $cdcampo = str_pad(substr($boleto->getCdbarrapag(), 19, 25), 25, '0', STR_PAD_LEFT);
            } else {

                $linhaBarra = '';
                $dslinhabarrapag = $boleto->getDslinhbarpag();
                for ($i = 0; $i < strlen($boleto->getDslinhbarpag()); $i++) {
                    if (in_array($dslinhabarrapag[$i], array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')))
                        $linhaBarra .= $dslinhabarrapag[$i];
                }

                $cdbanco = str_pad(substr($linhaBarra, 0, 3), 3, '0', STR_PAD_LEFT);
                $cdmoeda = str_pad(substr($linhaBarra, 3, 1), 1, '0', STR_PAD_LEFT);
                $cddigito = str_pad(substr($linhaBarra, 32, 1), 1, '0', STR_PAD_LEFT);
                $valor = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
                $cdcampo = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
            }
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha .= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha .= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha .= $cddigito;

            //Valor do Documento (Cd. Barra) - Posição: 23 a 36 - Picture: 9(10)
            $linha .= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha .= $cdcampo;

            //Nome do favorecido - Posição: 62 a 91 - Picture: X(30)
            $linha .= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30, ' ', STR_PAD_RIGHT);

            //Data vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha .= $boleto->getDtvenbolpag()->format('dmY');

            //Valor previsto do pagto - Posição: 100 a 114 - Picture: 9(13)V9(2)
            if ($boleto->getCdbarrapag()){
              $linha.= str_pad(substr($boleto->getCdbarrapag(), 10, 9), 15,'0',STR_PAD_LEFT);
              $vrCodBarras = (int)(substr($boleto->getCdbarrapag(), 10, 9)) / 100;
              if ($boleto->getVrfinalbpag() - $vrCodBarras > 0) {
                  $vrdesconto = 0;
                  $vracrescimo = $boleto->getVrfinalbpag() - $vrCodBarras;
              }
              else {
                  $vrdesconto = $vrCodBarras - $boleto->getVrfinalbpag();
                  $vracrescimo = 0;
              }
           }
           else {
               $vrdesconto = 0;
               $vracrescimo = 0;
               $linha.= str_pad(str_replace(array('.',','),'',
                   number_format($boleto->getVrfinalbpag(),2)
               ),15,'0',STR_PAD_LEFT);
           }

            //Valor do Desconto - Posição: 115 a 129 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($vrdesconto,2)
            ),15,'0',STR_PAD_LEFT);

            //Valor do Acrescimo - Posição: 130 a 144 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($vracrescimo,2)
            ),15,'0',STR_PAD_LEFT);

            //Data do pagamento - Posição: 145 a 152 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do pagto - Posição: 153 a 167 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'',
                number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Zeros - Posição:  168 a 182 - Picture: 9(15)
            $linha.= str_repeat('0', 15);

            //Seu número - Posição: 183 a 202 - Picture: X(20)
            $linha.= str_pad(trim($boleto->getCdfilial()).trim($boleto->getNrlanbolpag()),20,' ', STR_PAD_RIGHT );

            //Nosso número - Posição: 203 a 222 - Picture: X(20)
            if($this->remessa->getIdopecnabpag() == self::EXCLUSAO)
                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getNrretcnabpag(),0,20)),20,' ',STR_PAD_RIGHT);
            else $linha.= str_repeat(' ', 20);

            //Brancos - Posição: 223 a 224 - Picture: 9(2)
            $linha.= '09';

        }

        $linha = str_pad($linha, 240);

        return $linha."\r\n";
    }

    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro detalhe segmento B #########################//
     * //################################################################################//
     */
    private function geraSegmentoB(Boletpagar $boleto){

        $linha = '';

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_DETALHE;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
        $linha.= 'B';

        //BRANCOS - Posição: 15 a 17 - Picture: 9(03)
        $linha.= str_repeat(' ', 3);

        //Tipo de incrição do fornecedor - Posição: 18 a 18 - Picture: 9(01)
        if($boleto->getIdtpicnabbop()) {
            if ($boleto->getIdtpicnabbop() == 'F')
                $tipoInsc = '1'; //CPF
            else {
                if ($boleto->getIdtpicnabbop() == 'j')
                    $tipoInsc =  '2'; //CGC
                else
                    $tipoInsc = '3'; //Outros
            }
        }
        else {
            if ($boleto->getForneced()->getIdtpijurforn() == 'F')
                $tipoInsc = '1'; //CPF
            else {
                if ($boleto->getForneced()->getIdtpijurforn() == 'J')
                    $tipoInsc = '2'; //CGC
                else
                 $tipoInsc = '3'; //Outros
            }
        }
        $linha.= $tipoInsc;

        //Código do CPF/CNPJ - Posição: 19 a 32 - Picture: 9(14)
        if ($boleto->getNrinscnabbop())
           $linha.= str_pad(substr($boleto->getNrinscnabbop(), 0, 14), 14,'0',STR_PAD_LEFT);
        else
            $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);

        //Nome do logradouro do favorecido - Posição: 33 a 62 - Picture: X(30)
        $endforn = $boleto->getForneced()->getEndefornecobr();
        
        if(! $endforn)
            $endforn = $boleto->getForneced()->getEndeforneprinc();
        $linha.= str_pad(substr(Util::campoCnabValido($endforn->getDsendeforn()),0,30),30,' ',STR_PAD_RIGHT);

        //Número do imóvel do favorecido - Posição: 63 a 67 - Picture: 9(05)
        $linha.= str_repeat('0', 5);

        //Complemento do imóvel - Posição: 68 a 82 - Picture: X(15)
        $linha.= str_repeat(' ', 15);

        //Bairro do favorecido - Posição: 83 a 97 - Picture: 9(15)
        $linha.= str_pad(substr(Util::campoCnabValido($endforn->getNmbairforn()),0,15),15,' ',STR_PAD_RIGHT);

        //Município do favorecido - Posição: 98 a 117 - Picture: X(20)
        $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($endforn->getSgestado(), $endforn->getCdpais(), $endforn->getCdmunicipio());
        $linha.= str_pad(substr(Util::campoCnabValido($municipio->getNmmunicipio())
            ,0,20),20,' ',STR_PAD_RIGHT);

        //CEP - Posição: 118 a 122 - Picture: 9(05)
        if($endforn->getNrcepforn()){
            $linha.= str_pad(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),0,5),5,'0');
        }else{
            $linha.= str_repeat('0',5);
        }

        //Complemento do CEP - Posição: 123 a 125 - Picture: X(03)
        if(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3)){
            $linha.= substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3);

        }else{
            $linha.= str_repeat(' ',3);
        }

        //Estado - Posição: 126 a 127 - Picture: X(02)
        $linha.= str_pad(substr(Util::campoCnabValido($endforn->getSgestado()),0,2),2,' ',STR_PAD_RIGHT);

        //Data do vencimento (Nominal) - Posição: 128 a 135 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');

        //Valor do documento (Nominal) - Posição: 136 a 150 - Picture: 9(13)V9(2)
        $linha.= str_pad(str_replace(array('.',','),'',
            number_format($boleto->getVrfinalbpag(),2)
        ),15,'0',STR_PAD_LEFT);

        //Valor do abatimento - Posição: 151 a 165 - Picture: 9(13)V9(2)
        $linha.= str_repeat('0', 15);

        //Valor do desconto - Posição: 166 a 180 - Picture: 9(13)V9(2)
        $linha.= str_repeat('0', 15);

        //Valor da mora - Posição: 181 a 195 - Picture: 9(13)V9(2)
        $linha.= str_repeat('0', 15);

        //Valor da multa - Posição: 196 a 210 - Picture: 9(13)V9(2)
        $linha.= str_repeat('0', 15);

        //Codigo do Favorecido - Posição: 211 a 225 - Picture: X(15)
        $linha.= str_repeat(' ', 15);

        //Brancos - Posição: 226 a 240 - Picture: X(15)
        $linha.= str_repeat(' ', 15);

        return $linha."\r\n";
    }
    
    /**
     * @param Boletpagar $boleto
     * @return string
     * @throws Exception
     * @internal param $cdbanco
     * //################################################################################//
     * //################################################################################//
     * //#########################  Registro detalhe segmento B #########################//
     * //################################################################################//
     */
    private function geraSegmentoJ52(Boletpagar $boleto){

        $linha = '';

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha.= self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha .= str_pad($this->num_lote, 4, '0', STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_DETALHE;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha .= str_pad(++$this->seqRegLote, 5, '0', STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
        $linha.= 'J';
        
        //Uso Exclusivo FEBRABAN/CNAB - Posição: 15 a 15 - Picture: X(01)
        $linha.= str_repeat(' ', 1);
        
        //Código de Movimento Remessa - Posição: 16 a 17 - Picture: 9(02)
        if ($this->remessa->getIdopecnabpag() == 'E')
           $linha .= '99'; //Exclusão
        else {
            if ($this->remessa->getIdopecnabpag() == 'I')
                $linha .= '00'; //Inclusão
            else
                $linha .= '09'; // Bloqueio
        }
        
        //
        $linha.= '52';
        
        /*
         * #### PAGADOR ####
         */
        //Tipo de Inscrição Pagador - Posição: 20 a 20 - Picture: 9(1)
        $linha.= $this->tipoinscricao;
    
        //CPF/CNPJ do Pagador - Posição: 21 a 35 - Picture: 9(15)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
        
        //Nome do Pagador - Posição: 36 a 75 - Picture: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
        
        /*
         * #### Beneficiário ####
         */
        //Tipo de Inscrição Beneficiário - Posição: 76 a 76 - Picture: 9(01)
        if(strlen(Util::campoCnabValido($boleto->getForneced()->getNrinsjurforn())) == 11){
            $linha.= self::TIPO_INS_JUR_CPF;
        }else{
            $linha.= self::TIPO_INS_JUR_CNPJ;
        }
        
        //CPF/CNPJ do Beneficiário - Posição: 77 a 91 - Picture: 9(01)
        $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),15,'0', STR_PAD_LEFT);

        //Nome do Beneficiário - Posição: 92 a 131 - Picture: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,40)),40);
        
        /*
         * #### SACADOR ####
         */
        //Tipo de Inscrição Sacador - Posição: 132 a 132 - Picture: 9(1)
        $linha.= $this->tipoinscricao;

        //CPF/CNPJ do Sacador - Posição: 133 a 147 - Picture: 9(15)
        $linha.= str_pad(substr($this->inscricaojuridica,0,14),15,'0', STR_PAD_LEFT);
        
        //Nome do Sacador - Posição: 148 a 187 - Picture: X(40)
        $linha.= str_pad(Util::campoCnabValido(substr($this->nomeempresa,0,40)),40);
        
        //Filler - Posição: 188 a 240 - Picture: X(53)
        $linha.= str_repeat(' ',53);

        return $linha . "\r\n";
    }

    /**
     *#################################################################//
    //##################  Registro Trailler de Lote ###################//
    //#################################################################//
     */
    public function geraTraillerLote($somavalores = null)
    {

        $this->qtdRegiLote++; //Qtde de registro do lote
        $this->qtdRegiArq++; //Quantidade de registro do arquivo

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha = self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote do serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha.= str_pad(($this->num_lote), 4,'0',STR_PAD_LEFT);

        //Tipo de registro = Registro Trailer do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_TRAILLER_LOTE;

        //Brancos - Posição: 9 a 17 - Picture: X(9)
        $linha.= str_repeat(' ', 9);

        //Total qtd Registro do lote - Posição: 18 a 23 - Picture: 9(6)
        $linha.= str_pad($this->qtdRegiLote, 6, '0', STR_PAD_LEFT);

        //Valor do pagto - Posição: 24 a 41 - Picture: 9(16)V9(2)
        $linha.= str_pad(str_replace(array('.',','),'',
            number_format($this->vrPagtLote,2)
        ),18,'0',STR_PAD_LEFT);

        //Zeros - Posição: 42 a 59 - Picture: 9(18)
        $linha.= str_repeat('0', 18);

        //Brancos - Posição: 60 a 65 - Picture: X(6)
        $linha.= str_repeat('0', 6);

        //Brancos - Posição: 66 a 230 - Picture: X(165)
        $linha.= str_repeat(' ', 165);

        //Ocorrências - Posição: 231 a 240 - Picture: X(10)
        $linha.= str_repeat(' ', 10);

        return $linha."\r\n";

    }

    public function geraTrailerArquivo()
    {
        //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
        $linha = self::CODIGO_COMPENSACAO_BRASIL;

        //Código do lote do serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha.= self::LOTE_DE_SERVICO_TRAILLER_ARQUIVO;

        //Tipo de registro = registro trailer de arquivo - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::TIPO_REGISTRO_TRAILLER_ARQUIVO;

        //Brancos - Posição: 9 a 17 - Picture: X(9)
        $linha.= str_repeat(' ', 9);

        //Total Qtde de lotes do arquivo - Posição: 18 a 23 - Picture: 9(6)
        $linha.= str_pad($this->num_lote, 6,'0',STR_PAD_LEFT);

        //Total de Qtde de registro do arquivo - Posição: 24 a 29 - Picture: 9(6)
        $linha.= str_pad(++$this->qtdRegiArq, 6,'0',STR_PAD_LEFT);

        //Brancos - Posição: 30 a 35 - Picture: 9(6)
        $linha.= str_repeat('0', 6);

        //Brancos - Posição: 36 a 240 - Picture: X(205)
        $linha.= str_repeat(' ', 205);

        return $linha."\r\n";
    }

    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Bradesco
     * @throws Exception
     * @internal param $remessa
     * @return mixed
     */
    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null)
    {
        try{

            $file = fopen($caminhoArquivo.'.txt', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);

            // Agrupar boletos por tipo de baixa e por banco favorecido
            $boletos = self::agrupaBoletos($boletos, $file);

            $idbaixaAtual = ' '; //Utilizado para controle de geração de header de lote

            $cdbancoaux = ' ';

            $boTestar = true;

            $boletosAux = $boletos;

            $boTrailerLoteGerado = false;

            //Percorre as formas de pagamento
            /** @var $boleto Boletpagar  */
            foreach($boletos as $boleto){

                $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag());

                $boTrailerLoteGerado = false;

                /** @var $boletoAux Boletpagar */
                $boletoAux = next($boletosAux);

                $banco = self::getBanco($boleto, $boleto->getTpbaixapag()->getIdbaixautilp());
                
                $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();

                if (in_array($idbaixautilp, array('B', 'C', 'D', 'T', 'S', 'O'))) { //ROTINAS GERAIS

                    if (($idbaixaAtual != $idbaixautilp) or ($idbaixautilp == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO and
                            $cdbancoaux != $banco and $boTestar)
                    ) {

                        fprintf($file, "%s", self::geraHeaderLote($banco, $boleto));
                        $idbaixaAtual = $idbaixautilp;
                        $cdbancoaux = $banco;

                        if ($idbaixautilp == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO)
                            if ($banco != self::CODIGO_COMPENSACAO_BRASIL)
                                $boTestar = false;
                    }
                    $linha = self::geraSegmentoAJ($boleto, $banco);
                    fprintf($file, "%s", $linha);

                    if (substr($linha, 13, 1) == 'A')
                        fprintf($file, "%s", self::geraSegmentoB($boleto));
                        
                    if (substr($linha, 13, 1) == 'J')
                        fprintf($file, "%s", self::geraSegmentoJ52($boleto));
                }
                if ( $boletoAux and(($idbaixaAtual != $boletoAux->getTpbaixapag()->getIdbaixautilp()) or
                    $boletoAux->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO and
                    substr($boletoAux->getCdbarrapag(), 0, 3) != $cdbancoaux and
                    $boTestar)){

                    fprintf($file, "%s", self::geraTraillerLote());
                    $boTrailerLoteGerado = true;
                }
            }
            if(! $boTrailerLoteGerado)
                fprintf($file, "%s",self::geraTraillerLote());


            fprintf($file, "%s",self::geraTrailerArquivo());
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }

    /**
     * @param array|Boletpagar $boletos
     * Agrupa os boletos por tipo de baixa e código do banco
     *
     * @return array
     */
    private function agrupaBoletos($boletos = array()){

        $boletosAgroup = array();

        /** @var $boleto Boletpagar */
        foreach($boletos as $boleto){

            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']);

            $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();

            $boletosAgroup[$idbaixautilp]
                          [self::getBanco($boleto,$idbaixautilp)] = $boleto;
        }

        $boletos = array();
        ksort($boletosAgroup);

        foreach($boletosAgroup as $idBaixas){
            foreach($idBaixas as $boleto)
                $boletos[] = $boleto;
        }
        return $boletos;
    }

    /**
     * @param $cdbanco
     * @param Boletpagar $boleto
     */
    private function getFormaPagto($cdbanco, $boleto){
        
        $idbaixa = $boleto->getTpbaixapag()->getIdbaixautilp();

        if($idbaixa == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG)
           return '10';
        else if ($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS)
            return '11';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_GPS)
            return '17';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_DARF_SIMPLES)
            return '18';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL)
            return '16';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_DARJ)
            return '21';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_FGTS)
            return '99';
        else if($idbaixa == Tipobaixapag::TIPO_BAIXA_GARE)
            return '22';
        else{
            if($idbaixa == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($cdbanco == self::CODIGO_COMPENSACAO_BRASIL)
                    return '30'; //Liquidação de Títulos do Próprio Banco
                else
                    return '31'; //Pagamento de Títulos de Outros Bancos
            }else{
                if($cdbanco == self::CODIGO_COMPENSACAO_BRASIL)
                    return '01'; //Crédito em conta corrente
                else{
                    return '03'; //DOC/TED
                }
            }
        }

    }
}